@@ -16,7 +16,7 @@
 ######################################################################################################################################
 ######################################################################################################################################
 
-MiSSE <- function(phy, f=1, turnover=c(1,2), eps=c(1,2), condition.on.survival=TRUE, root.type="madfitz", root.p=NULL, sann=FALSE, sann.its=10000, bounded.search=TRUE, max.tol=.Machine$double.eps^.50, starting.vals=NULL, turnover.upper=10000, eps.upper=3, trans.upper=100, restart.obj=NULL, ode.eps=0){
+MiSSE <- function(phy, f=1, turnover=c(1,2), eps=c(1,2), condition.on.survival=TRUE, root.type="madfitz", root.p=NULL, sann=FALSE, sann.its=10000, bounded.search=TRUE, max.tol=.Machine$double.eps^.50, starting.vals=NULL, turnover.upper=10000, eps.upper=3, trans.upper=100, restart.obj=NULL, ode.eps=0, dt.threads=1){
     
     ## Temporary fix for the current BUG:
     if( !is.null(phy$node.label) ) phy$node.label <- NULL
@@ -33,6 +33,8 @@ MiSSE <- function(phy, f=1, turnover=c(1,2), eps=c(1,2), condition.on.survival=T
         }
     }
 
+    setDTthreads(threads=dt.threads)
+
     if(!root.type == "madfitz" & !root.type == "herr_als"){
         stop("Check that you specified a proper root.type option. Options are 'madfitz' or 'herr_als'. See help for more details.", call.=FALSE)
     }
@@ -288,16 +290,18 @@ OrganizeDataMiSSE <- function(phy, f, hidden.states){
     colnames(tmp.df) <- c("RootwardAge", "TipwardAge", "BranchLength", "FocalNode", "DesNode", "comp", paste("compD", 1:ncol(compD), sep="_"), paste("compE", 1:ncol(compE), sep="_"))
     dat.tab <- as.data.table(tmp.df)
     setkey(dat.tab, DesNode)
+    cols <- names(dat.tab)
     for (j in 1:(dim(compD)[2])){
-        dat.tab[data.table(c(1:nb.tip)), paste("compD", j, sep="_") := compD[,j]]
-        dat.tab[data.table(c(1:nb.tip)), paste("compE", j, sep="_") := compE[,j]]
+        #dat.tab[data.table(c(1:nb.tip)), paste("compD", j, sep="_") := compD[,j]]
+        set(dat.tab, 1:nb.tip, cols[6+j], compD[,j])
+        #dat.tab[data.table(c(1:nb.tip)), paste("compE", j, sep="_") := compE[,j]]
+        set(dat.tab, 1:nb.tip, cols[32+j], compE[,j])
     }
     return(dat.tab)
 }
 
 
-SingleChildProbMiSSE <- function(cache, compD, compE, start.time, end.time, x){
-    pars <- c(cache$lambda0A, cache$mu0A, cache$lambda0B, cache$mu0B, cache$lambda0C, cache$mu0C, cache$lambda0D, cache$mu0D, cache$lambda0E, cache$mu0E, cache$lambda0F, cache$mu0F, cache$lambda0G, cache$mu0G, cache$lambda0H, cache$mu0H, cache$lambda0I, cache$mu0I, cache$lambda0J, cache$mu0J, cache$lambda0K, cache$mu0K, cache$lambda0L, cache$mu0L, cache$lambda0M, cache$mu0M, cache$lambda0N, cache$mu0N, cache$lambda0O, cache$mu0O, cache$lambda0P, cache$mu0P, cache$lambda0Q, cache$mu0Q, cache$lambda0R, cache$mu0R, cache$lambda0S, cache$mu0S, cache$lambda0T, cache$mu0T, cache$lambda0U, cache$mu0U, cache$lambda0V, cache$mu0V, cache$lambda0W, cache$mu0W, cache$lambda0X, cache$mu0X, cache$lambda0Y, cache$mu0Y, cache$lambda0Z, cache$mu0Z, cache$q0, cache$hidden.states)
+SingleChildProbMiSSE <- function(cache, pars, compD, compE, start.time, end.time){
     yini <- c(E0A = compE[1], E0B = compE[2], E0C = compE[3], E0D = compE[4], E0E = compE[5], E0F = compE[6], E0G = compE[7], E0H = compE[8], E0I = compE[9], E0J = compE[10], E0K = compE[11], E0L = compE[12], E0M = compE[13], E0N = compE[14], E0O = compE[15], E0P = compE[16], E0Q = compE[17], E0R = compE[18], E0S = compE[19], E0T = compE[20], E0U = compE[21], E0V = compE[22], E0W = compE[23], E0X = compE[24], E0Y = compE[25], E0Z = compE[26], D0A = compD[1], D0B = compD[2], D0C = compD[3], D0D = compD[4], D0E = compD[5], D0F = compD[6], D0G = compD[7], D0H = compD[8], D0I = compD[9], D0J = compD[10], D0K = compD[11], D0L = compD[12], D0M = compD[13], D0N = compD[14], D0O = compD[15], D0P = compD[16], D0Q = compD[17], D0R = compD[18], D0S = compD[19], D0T = compD[20], D0U = compD[21], D0V = compD[22], D0W = compD[23], D0X = compD[24], D0Y = compD[25], D0Z = compD[26])
     times=c(start.time, end.time)
 
@@ -334,16 +338,18 @@ SingleChildProbMiSSE <- function(cache, compD, compE, start.time, end.time, x){
 }
 
 
-FocalNodeProbMiSSE <- function(cache, dat.tab, generations){
+FocalNodeProbMiSSE <- function(cache, pars, lambdas, dat.tab, generations){
     ### Ughy McUgherson. This is a must in order to pass CRAN checks: http://stackoverflow.com/questions/9439256/how-can-i-handle-r-cmd-check-no-visible-binding-for-global-variable-notes-when
     DesNode = NULL
     FocalNode = NULL
-
+    . = NULL
+    
+    gens <- data.table(c(generations))
     setkey(dat.tab, FocalNode)
-    CurrentGenData <- dat.tab[data.table(generations)]
-    tmp <- t(apply(CurrentGenData, 1, function(z) SingleChildProbMiSSE(cache, z[7:32], z[33:58],  z[2], z[1])))
+    CurrentGenData <- dat.tab[gens]
+    tmp <- t(apply(CurrentGenData, 1, function(z) SingleChildProbMiSSE(cache, pars, z[7:32], z[33:58],  z[2], z[1])))
     v.mat <- matrix(tmp[seq(1,nrow(tmp)-1,2),27:52] * tmp[seq(2,nrow(tmp),2),27:52], length(unique(CurrentGenData$FocalNode)), 26)
-    v.mat <- v.mat * matrix(c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z), length(unique(CurrentGenData$FocalNode)), 26, byrow=TRUE)
+    v.mat <- v.mat * matrix(lambdas, length(unique(CurrentGenData$FocalNode)), 26, byrow=TRUE)
     phi.mat <- matrix(tmp[seq(1,nrow(tmp)-1,2),1:26], length(unique(CurrentGenData$FocalNode)), 26)
     if(!is.null(cache$node)){
         if(which(generations == cache$node)){
@@ -355,9 +361,13 @@ FocalNodeProbMiSSE <- function(cache, dat.tab, generations){
     tmp.comp <- rowSums(v.mat)
     tmp.probs <- v.mat / tmp.comp
     setkey(dat.tab, DesNode)
+    rows <- dat.tab[.(generations), which=TRUE]
+    cols <- names(dat.tab)
     for (j in 1:(dim(tmp.probs)[2])){
-        dat.tab[data.table(c(generations)), paste("compD", j, sep="_") := tmp.probs[,j]]
-        dat.tab[data.table(c(generations)), paste("compE", j, sep="_") := phi.mat[,j]]
+        #dat.tab[data.table(c(generations)), paste("compD", j, sep="_") := tmp.probs[,j]]
+        set(dat.tab, rows, cols[6+j], tmp.probs[,j])
+        #dat.tab[data.table(c(generations)), paste("compE", j, sep="_") := phi.mat[,j]]
+        set(dat.tab, rows, cols[32+j], phi.mat[,j])
     }
     dat.tab[data.table(c(generations)), "comp" := tmp.comp]
     return(dat.tab)
@@ -365,15 +375,15 @@ FocalNodeProbMiSSE <- function(cache, dat.tab, generations){
 
 
 #Have to calculate root prob separately because it is not a descendant in our table. Could add it, but I worry about the NA that is required.
-GetRootProbMiSSE <- function(cache, dat.tab, generations){
+GetRootProbMiSSE <- function(cache, pars, lambdas, dat.tab, generations){
     ### Ughy McUgherson. This is a must in order to pass CRAN checks: http://stackoverflow.com/questions/9439256/how-can-i-handle-r-cmd-check-no-visible-binding-for-global-variable-notes-when
     FocalNode = NULL
-
+    gens <- data.table(c(generations))
     setkey(dat.tab, FocalNode)
-    CurrentGenData <- dat.tab[data.table(generations)]
-    tmp <- t(apply(CurrentGenData, 1, function(z) SingleChildProbMiSSE(cache, z[7:32], z[33:58],  z[2], z[1])))
+    CurrentGenData <- dat.tab[gens]
+    tmp <- t(apply(CurrentGenData, 1, function(z) SingleChildProbMiSSE(cache, pars, z[7:32], z[33:58],  z[2], z[1])))
     v.mat <- matrix(tmp[seq(1,nrow(tmp)-1,2),27:52] * tmp[seq(2,nrow(tmp),2),27:52], length(unique(CurrentGenData$FocalNode)), 26)
-    v.mat <- v.mat * matrix(c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z), length(unique(CurrentGenData$FocalNode)), 26, byrow=TRUE)
+    v.mat <- v.mat * matrix(lambdas, length(unique(CurrentGenData$FocalNode)), 26, byrow=TRUE)
     phi.mat <- matrix(tmp[seq(1,nrow(tmp)-1,2),1:26], length(unique(CurrentGenData$FocalNode)), 26)
     if(!is.null(cache$node)){
         if(which(generations == cache$node)){
@@ -403,6 +413,9 @@ DownPassMisse <- function(dat.tab, gen, cache, condition.on.survival, root.type,
     DesNode = NULL
     compE = NULL
 
+    pars <- c(cache$lambda0A, cache$mu0A, cache$lambda0B, cache$mu0B, cache$lambda0C, cache$mu0C, cache$lambda0D, cache$mu0D, cache$lambda0E, cache$mu0E, cache$lambda0F, cache$mu0F, cache$lambda0G, cache$mu0G, cache$lambda0H, cache$mu0H, cache$lambda0I, cache$mu0I, cache$lambda0J, cache$mu0J, cache$lambda0K, cache$mu0K, cache$lambda0L, cache$mu0L, cache$lambda0M, cache$mu0M, cache$lambda0N, cache$mu0N, cache$lambda0O, cache$mu0O, cache$lambda0P, cache$mu0P, cache$lambda0Q, cache$mu0Q, cache$lambda0R, cache$mu0R, cache$lambda0S, cache$mu0S, cache$lambda0T, cache$mu0T, cache$lambda0U, cache$mu0U, cache$lambda0V, cache$mu0V, cache$lambda0W, cache$mu0W, cache$lambda0X, cache$mu0X, cache$lambda0Y, cache$mu0Y, cache$lambda0Z, cache$mu0Z, cache$q0, cache$hidden.states)
+    lambda <- c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z)
+    
     nb.tip <- cache$nb.tip
     nb.node <- cache$nb.node
     TIPS <- 1:nb.tip
@@ -412,14 +425,14 @@ DownPassMisse <- function(dat.tab, gen, cache, condition.on.survival, root.type,
                 if(node %in% gen[[i]]){
                     cache$node <- node
                     cache$state <- state
-                    res.tmp <- GetRootProbMiSSE(cache=cache, dat.tab=dat.tab, generations=gen[[i]])
+                    res.tmp <- GetRootProbMiSSE(cache=cache, pars=pars, lambdas=lambda, dat.tab=dat.tab, generations=gen[[i]])
                     cache$node <- NULL
                     cache$state <- NULL
                 }else{
-                    res.tmp <- GetRootProbMiSSE(cache=cache, dat.tab=dat.tab, generations=gen[[i]])
+                    res.tmp <- GetRootProbMiSSE(cache=cache, pars=pars, lambdas=lambda, dat.tab=dat.tab, generations=gen[[i]])
                 }
             }else{
-                res.tmp <- GetRootProbMiSSE(cache=cache, dat.tab=dat.tab, generations=gen[[i]])
+                res.tmp <- GetRootProbMiSSE(cache=cache, pars=pars, lambdas=lambda, dat.tab=dat.tab, generations=gen[[i]])
             }
             compD.root <- res.tmp[c(28:53)]
             compE.root <- res.tmp[c(2:27)]
@@ -431,14 +444,14 @@ DownPassMisse <- function(dat.tab, gen, cache, condition.on.survival, root.type,
                 if(node %in% gen[[i]]){
                     cache$node <- node
                     cache$state <- state
-                    dat.tab <- FocalNodeProbMiSSE(cache, dat.tab, gen[[i]])
+                    dat.tab <- FocalNodeProbMiSSE(cache, pars=pars, lambdas=lambda, dat.tab, gen[[i]])
                     cache$node <- NULL
                     cache$state <- NULL
                 }else{
-                    dat.tab <- FocalNodeProbMiSSE(cache, dat.tab, gen[[i]])
+                    dat.tab <- FocalNodeProbMiSSE(cache, pars=pars, lambdas=lambda, dat.tab, gen[[i]])
                 }
             }else{
-                dat.tab <- FocalNodeProbMiSSE(cache, dat.tab, gen[[i]])
+                dat.tab <- FocalNodeProbMiSSE(cache, pars=pars, lambdas=lambda, dat.tab, gen[[i]])
             }
         }
     }
@@ -453,13 +466,13 @@ DownPassMisse <- function(dat.tab, gen, cache, condition.on.survival, root.type,
         }
         if(condition.on.survival == TRUE){
             if(root.type == "madfitz"){
-                lambda <- c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z)
+                #lambda <- c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z)
                 compD.root <- compD.root / sum(root.p * lambda * (1 - compE.root)^2)
                 #Corrects for possibility that you have 0/0:
                 compD.root[which(is.na(compD.root))] = 0
                 loglik <- log(sum(compD.root * root.p)) + sum(log(comp))
             }else{
-                lambda <- c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z)
+                #lambda <- c(cache$lambda0A, cache$lambda0B, cache$lambda0C, cache$lambda0D, cache$lambda0E, cache$lambda0F, cache$lambda0G, cache$lambda0H, cache$lambda0I, cache$lambda0J, cache$lambda0K, cache$lambda0L, cache$lambda0M, cache$lambda0N, cache$lambda0O, cache$lambda0P, cache$lambda0Q, cache$lambda0R, cache$lambda0S, cache$lambda0T, cache$lambda0U, cache$lambda0V, cache$lambda0W, cache$lambda0X, cache$lambda0Y, cache$lambda0Z)
                 compD.root <- (compD.root * root.p) / (lambda * (1 - compE.root)^2)
                 #Corrects for possibility that you have 0/0:
                 compD.root[which(is.na(compD.root))] = 0
